
/*
   Java client for the Palantir system.

   Copyright 2001-2004 David Santinoli
   Released under the GPL. See file COPYING for details.

   I chose to stick to the old event API, with the deprecated "action()", as
   not all browsers are Java 1.2-compliant (see NS/MacOS for a shameful
   example).

   Note: getPreferredSize and getMinimumSize "force" panel size to 320x240,
   so there's no need to scale. A more elegant solution would be to set
   c.weightx=c.weighty=1 in mainPanel (thus making the window resizable)
   but this seems to cause flicker on IE.

   TODO: event-driven accounting of control change;
   migration to JDK 1.4

*/

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.util.*;
import java.io.*;
import java.text.*;

public class pclient extends Applet implements Runnable  {

 private Image im = null;
 private String client_version;
 private byte imgbuf[];
 private int n, strs=0;
 private InputStream is;
 private PrintStream os;
 private volatile Thread mainthread=null;
 private Socket sock;
 private int frames, frame1;
 private int snapshotOnly;
 private long time0, time1;
 private Button start_button, control_button, grab_button, devices_button;
 private Checkbox devices_cbox;
 private GPanel mainPanel;
 private Label infoLabel;
 private Label controlLabel;
 private TextField textField;
 public Scrollbar xslider, yslider, zslider; /* XXX public is too much... */
 private NumberFormat nf = NumberFormat.getInstance();
 private int ctl_status;
 private deviceFrame devsFrame;
 private boolean ack = true;
 
 private class ctl_enum {
    public static final int DEQUEUED = 0;
    public static final int ASKING = 1;
    public static final int PENDING = 2;
    public static final int NOW = 3;
    };

 private class GPanel extends Panel {

    public Dimension getPreferredSize() {
       Dimension preferredSize;
       //System.out.println("getPreferredSize invoked");
       if(im==null) preferredSize = new Dimension(320+6,240+6);
       else preferredSize = new Dimension(im.getWidth(this)+6,im.getHeight(this)+6);
       //System.out.println("PreferredSize is "+preferredSize);
       return preferredSize;
       }

    /* getMaximumSize is apparently not invoked */

    public Dimension getMaximumSize() {
       System.out.println("getMaximumSize Invoked");
       return new Dimension(320+6,240+6);
       }

    public Dimension getMinimumSize() {
       System.out.println("getMinimumSize Invoked");
       return new Dimension(320+6,240+6);
       }

    public void update(Graphics g) {
       paint(g);
       }

    public void paint(Graphics g) {
       //System.out.println("Now in img paint()");
       if(im!=null)
          // should precompute position
          g.drawImage(im,
            3,3,getSize().width-6,getSize().height-6,null);
            //3,3,320,240,null);
            //System.out.println("scaled to "+(getSize().width-6)+" "+(getSize().height-6));
       //System.out.println("Now leaving img paint()");
       }

    }


 public void update(Graphics g) {
    paint(g);
    }


 public void paint(Graphics g) {
    mainPanel.update(g);
    }


 public void launch() {
    if(mainthread == null) {
       if(connect() && read_headers()) {
          infoLabel.setText("");
          controlLabel.setText("");
          mainthread = new Thread(this);
          frame1=frames=0;
          start_button.setLabel("Stop");
          devices_cbox.setEnabled(devsFrame.needDevBox());
          time1=time0=System.currentTimeMillis();
          loseControl();
          mainthread.start();
          }
       }
    else mainthread=null;
    }


 public boolean action(Event e, Object obj) {
    String arg = e.arg.toString();
    if(e.target==start_button) {
       controlLabel.setText("");
       if(mainthread==null) launch();
       else mainthread = null;
       }
    else if(e.target==control_button && mainthread!=null && ctl_status==ctl_enum.DEQUEUED) {
       controlLabel.setText("waiting to take control...");
       ctl_status=ctl_enum.ASKING;
       }
    else if(e.target==grab_button) {
       StringBuffer urlbuf = new StringBuffer();
       urlbuf.append("http://"+this.getCodeBase().getHost());
       urlbuf.append(":3000/?mode=single");
       try {
          URL url = new URL(new String(urlbuf));
          this.getAppletContext().showDocument(url,"Grabbed image");
          }
       catch(Exception ex) {
          System.out.println("Exception in grabbing: " + ex);
          }
       }
    else if(e.target==textField && mainthread!=null) {
       textField.setEnabled(false);
       }
    else if(e.target==devices_cbox) {
       devsFrame.setVisible(devices_cbox.getState());
       }
    return true;
    }


 public void stop() {
    System.out.println("Applet stopped.");
    try {
       if(is!=null) { is.close(); is=null; }
       } catch (Exception e) { System.out.println("closing stream: " + e); }
    }


 public void start() {
    client_version="pclient/ack 2.3 (Java)";

    if(getParameter("ack")!=null && getParameter("ack").equals("0")) {
       ack = false;
       client_version="pclient/noack 2.3 (Java)";
       }
    if(getParameter("autostart")!=null && getParameter("autostart").equals("1"))
{
       launch();
       }
    }


 public void run() {
    long ms;
    Thread thisThread = Thread.currentThread();
    while(mainthread == thisThread) {
       // gc here seems necessary at least on NS/Linux...
       System.gc();
       if(loadimage()==false) {
          infoLabel.setText("Error receiving data");
          break;
          }
       mainPanel.repaint();
       
       try {
          //mainthread.yield();
          mainthread.sleep(10);
          }
       catch (Exception e) {
          System.out.println("Exception while sleeping:" + e);
          }
       
       } // while

    mainthread=null;
    System.out.println("Main thread exited.");
    ms=System.currentTimeMillis()-time0;
    disconnect();
    if(snapshotOnly==1)
       infoLabel.setText("Snapshot mode (server busy)");
    else if(frames!=0) infoLabel.setText(frames + " frames in " + nf.format(ms/1000.0) + " s ("+ nf.format((frames*1000.0)/ms)+" f/s)");
    start_button.setLabel("Start");
    }


 private boolean read_headers() {
    String s;
    int n=0;

    devsFrame.flushDevices();
    try {
       do {
          s=readLine();
          if(s.startsWith("X-Defined-Sensors: ")) {
             int devices=Integer.parseInt(s.substring(19));
             System.out.println(devices + " devices defined");
             devsFrame.initDevices(devices);
             }
          else if(s.startsWith("X-Sensor-Define: ")) {
             devsFrame.addDevice(s.substring(17),this);
             }
          else if(s.equals("X-Snapshot: 1")) {
             snapshotOnly=1;
             break;
             }
          } while(s.length()>0);
       }
    catch (Exception e) {
       System.out.println("read_headers failed: "+e);
       e.printStackTrace();
       infoLabel.setText("Corrupted data headers.");
       return false;
       }
    return true;
    }


 private boolean connect() {
    String host;
    int port;

    snapshotOnly=0;
    if(this.getParameter("server")!=null)
       host = new String(this.getParameter("server"));
    else
       host = new String(this.getCodeBase().getHost());

    if(this.getParameter("port")!=null)
       port = Integer.parseInt(this.getParameter("port"));
    else port = 3000;

    infoLabel.setText("Connecting to " + host + ":" + port + "...");
    System.out.println("Connecting to " + host + ":" + port + "...");
    strs = 0;
    try {
       sock = new Socket(host,port);
       is=sock.getInputStream();
       os = new PrintStream(sock.getOutputStream());
       os.print("GET /\r\n");
       os.print("User-Agent: " + client_version + "\r\n\r\n");
       os.flush();
       }
    catch (Exception e) {
       System.out.println("connect() failed: "+ e);
       infoLabel.setText("Unable to connect to " + host + ".");
       sock=null;
       }       
    return !(sock==null);
    }


 private void disconnect() {
    try {
       if(sock!=null) { is.close(); sock.close(); /*sock=null;*/ }
       }
    catch (Exception e) {
       System.out.println("Exception closing stream: " + e);
       }
    loseControl();
    }


 public void init() {
    Panel infoPanel;
    Panel rightPanel;
    Panel ctrlPanel;
    Panel cboxPanel;
    Panel cameraPanel;
    BorderLayout bl, bl1;
    GridBagConstraints c = new GridBagConstraints();
    GridBagLayout gridbag = new GridBagLayout();
    n=0;
    im=null;
    imgbuf = new byte[100000];

    nf.setMaximumFractionDigits(1);
    nf.setMinimumFractionDigits(1);

    System.out.println(client_version + " loaded and ready to start.");
    setLayout(bl1 = new BorderLayout());
    bl1.setHgap(3);
    add("Center",cameraPanel = new Panel());

    cameraPanel.setLayout(gridbag);

    mainPanel = new GPanel();
    c.weightx=1;
    c.weighty=1;
    c.fill=GridBagConstraints.BOTH;
    c.insets= new Insets(0,0,0,0);
    //c.insets= new Insets(5,5,5,5);
    c.gridx=0; c.gridy=0;
    gridbag.setConstraints(mainPanel,c);
    cameraPanel.add(mainPanel);

    c.fill=GridBagConstraints.VERTICAL;
    yslider = new Scrollbar(Scrollbar.VERTICAL,127,20,0,275);
    c.weightx=0.0;
    c.weighty=0.0;
    c.insets= new Insets(0,0,0,0);
    c.gridx=1; c.gridy=0;
    gridbag.setConstraints(yslider,c);
    cameraPanel.add(yslider);

    c.fill=GridBagConstraints.HORIZONTAL;
    xslider = new Scrollbar(Scrollbar.HORIZONTAL,127,20,0,275);
    c.gridx=0; c.gridy=1;
    gridbag.setConstraints(xslider,c);
    cameraPanel.add(xslider);

    zslider = new Scrollbar(Scrollbar.HORIZONTAL,127,20,0,275);
    add("South", infoPanel = new Panel());
    infoPanel.setLayout(new BorderLayout());
    infoPanel.add("North", infoLabel = new Label());
    infoPanel.add("Center", controlLabel = new Label());
    infoPanel.add("South", textField = new TextField("",30));

    add("East", rightPanel = new Panel());
    rightPanel.setLayout(new BorderLayout());

    cboxPanel = new Panel();
    cboxPanel.setLayout(new FlowLayout());
    cboxPanel.add(devices_cbox = new Checkbox("Devices"));
    devices_cbox.setEnabled(true);

    ctrlPanel = new Panel();
    bl = new BorderLayout();
    bl.setVgap(5);
    bl.setHgap(3);
    ctrlPanel.setLayout(bl);
    ctrlPanel.add("Center", start_button = new Button("Start"));
    ctrlPanel.add("South", control_button = new Button("Control"));
    //ctrlPanel.add("North", grab_button = new Button("Grab"));

    rightPanel.add("Center", cboxPanel);
    rightPanel.add("North", ctrlPanel);
    rightPanel.add("South", zslider);

    devsFrame = new deviceFrame();
    devsFrame.openDeviceFrame();

    loseControl();
    }


 public Image makeJPEG(byte[] JPEGArray, int len, ImageObserver imo) {
    Toolkit tk=Toolkit.getDefaultToolkit();
    Image tmp;
    boolean err=false;

    //System.out.println("Now in makeJPEG...");
    tmp=tk.createImage(JPEGArray,0,len);
    while(!err & !tk.prepareImage(tmp,-1,-1,imo)) {
       err=((this.checkImage(tmp,imo)&imo.ERROR)!=0);
       if(err) {
          mainthread=null;
          return null;
          }
       } 
    strs=0;
    return tmp;
    }


 /* note: terminating \r\n is discarded */
 private String readLine() throws Exception {
    char readbuf[];
    boolean endseq=false;
    int b,ll=0,i;
    String s="", ss="";

    //System.out.println("Now in readLine");
    readbuf = new char[1000];
    try {
       while((b=is.read())!=-1 && ll<1000) {
          readbuf[ll++]=(char)b;
          if(b==0x0d) {
             endseq=true;
             continue;
             }
          if(b==0x0a && endseq) {
             ll-=2;
             break;
             }
          endseq=false;
          }
    
          s = new String(readbuf,0,ll);

          if(b==-1) throw new Exception();
          if(ll>250) {
             throw new Exception("Unexpectedly long string");
             }
          if(strs++>30) {
             System.out.println("Too many malformed strings");
             throw new Exception();
             }
          }
       catch (Exception e) {
          System.out.println("exception in readLine: "+e);
          throw e;
          }
    return s;
    }


 private void addParam(StringBuffer s, String par) {
    if(s.length()>0) s.append("; ");
    s.append(par);
    }


 private void gainControl() {
    controlLabel.setText("Now in control");
    ctl_status=ctl_enum.NOW;
    devsFrame.activateControls();
    }


 private void loseControl() {
    ctl_status=ctl_enum.DEQUEUED;
    devsFrame.deactivateControls();
    }


 private boolean loadimage() {
    int i,idx,l=0,tot,fpr=0,ctl_time;
    String s;
    long tmpTime;
    StringBuffer params = new StringBuffer();

    n++;
    try {
       if(snapshotOnly==0) do {
          s=readLine();
          } while (!s.equals("--mp-boundary"));
          //System.out.println("Found boundary...");

       do {
          s = readLine();
          if(s.length()==0) break;

          if(s.startsWith("X-Index: "))
             idx=Integer.parseInt(s.substring(2+s.indexOf(':')));
          else if(s.startsWith("Content-Length:"))
             l=Integer.parseInt(s.substring(2+s.indexOf(':')));
          else if(s.startsWith("Content-Type:")) ;
          else if(s.startsWith("X-Ctl-Delay: ")) {
             ctl_time=Integer.parseInt(s.substring(13));
             if(ctl_status==ctl_enum.PENDING) {
                if(ctl_time==-1) {
                   controlLabel.setText("Control functions not available.");
                   ctl_status=ctl_enum.DEQUEUED;
                   }
                else controlLabel.setText("Will take control in " + ctl_time + " s.");
                }
             }
          else if(s.startsWith("X-Ctl-Expiry: ")) {
             ctl_time=Integer.parseInt(s.substring(14));
             if(ctl_time==0) {
                controlLabel.setText("Another client is now in control.");
                loseControl();
                }
             else if(ctl_status==ctl_enum.PENDING) {
                gainControl();
                controlLabel.setText("Will keep control for at least " + ctl_time + " s.");
                }
             }
          else if(s.startsWith("X-Sensor-Change: ")) {
             //System.out.println(s);
             StringTokenizer st = new StringTokenizer(s,"; ");
             while(st.hasMoreTokens()) {
                String tok = new String (st.nextToken());
                if(tok.charAt(0)=='c') devsFrame.setDevice(tok);
                }
             }
          } while(snapshotOnly==0);

       //System.out.println("Length: "+l);

       tot = 0;
       while(tot<l) {
          //System.out.println("reading from stream...");
          tot+=is.read(imgbuf,tot,l-tot);
          //System.out.println("  finished reading.");
          //System.out.println("tot="+tot);
          }

       if(ctl_status==ctl_enum.NOW) {
          devsFrame.setParams(params);
          }

       if(!textField.isEnabled() && textField.getText().length()>0) {
          addParam(params,new String(
             "t="+textField.getText().replace(';','%') ) );
          textField.setText("");
          textField.setEnabled(true);
          }
       if(ctl_status==ctl_enum.ASKING) {
          addParam(params,new String("k=1"));
          ctl_status=ctl_enum.PENDING;
          }
       if(ack) {
          if(params!=null) os.print(params);
          os.print("\r\n");
          os.flush();
          }
       im=makeJPEG(imgbuf,l,null);
       }
    catch (Exception e) {
       System.out.println("loadimage(): " + e);
       return false;
       }
    frames++;
    //controlLabel.setText("Frame " + idx);


       tmpTime=System.currentTimeMillis();
       if(tmpTime-time1>=3000) {
          infoLabel.setText(
             nf.format( (1000.0*(frames-frame1))/(tmpTime-time1) ) + " f/s");
          time1=tmpTime;
          frame1=frames;
          }

    return true;
    }

}
