/***************************************************************************
                          Servers.cpp  -  description
                             -------------------
    begin                : lun mag 10 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldaniele@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Servers.h"

#include <stdio.h>
#include <string.h>

static const char rcsid[]="$Id: Servers.cpp,v 1.1 2004/07/01 19:57:01 santinod Exp $";

ServerType  gServer[MAX_SERVERS];
int         gDefaultServer;
bool        gDefaultServerAutoconnect;
ServerType  gTempServer;

static FILE *fp;
static bool gModified;
static char gDefaultFilename[] = "./pclient.ini";
static char buffer[256];

inline int ini_read()
{
 int s;
 int i;
 char *c;
 char string[512];

 //Clear server array...
 for(i=0;i<MAX_SERVERS;i++)
 {
  gServer[i].name[0]='\0';
  gServer[i].port[0]='\0';
  gServer[i].label[0]='\0';
  gServer[i].autoconnect = false;
 }

 //search for [global]
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {
   perror("ini_read()");
   return -1;
  }
  if (buffer[0] == ';') continue;
  if (strstr(buffer,"[global]") != NULL) break;
 }

 //search for server name and port
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {
   perror("ini_read()");
   return -1; 
  }
  if ( (c = strstr(buffer,"defaultserver= ")) != NULL) break;
 }

 //get server name
 c = &c[15];
 for(i=0;c[i]!=':' && c[i]!='\n' && c[i]!=' ' && c[i]!='\r';i++) gTempServer.name[i]=c[i];
 gTempServer.name[i] = '\0';
 if (c[i]==':') //there is a port
 {
  //get port
  c = &c[i+1];
  for(i=0;c[i]!='\n' && c[i]!=' ' && c[i]!='\r';i++) gTempServer.port[i]=c[i];
  gTempServer.port[i]='\0';
 } else gTempServer.port[0]='\0';

 //set label to null
 gTempServer.label[0] = '\0';

 //search for autoconnect status
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {perror("ini_read()");
   return -1;
  }
  if ( (c = strstr(buffer,"autoconnect= ")) != NULL) break;
 }
 c=&c[13];
 if (*c == '1') gDefaultServerAutoconnect = true; else gDefaultServerAutoconnect = false;

 //search for [serverlist]
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {
   perror("ini_read()");
   return -1; 
  }
  if (strstr(buffer,"[serverlist]") != NULL) break;
 }

 //search for server name and port
 for(s=0;s<MAX_SERVERS;s++)
 {
  for(;;)
  {if (fgets(buffer,256,fp)==NULL) return 0; //end of file
   sprintf(string,"server%1d= ",s);
   if ( (c = strstr(buffer,string)) != NULL) break;
  }
 
  //get server name
  c = &c[9];
  for(i=0;c[i]!=':' && c[i]!='\n' && c[i]!=' ' && c[i]!=',';i++) gServer[s].name[i]=c[i];
  gServer[s].name[i] = '\0';
  if (c[i]==':') //there is a port
  {
   //get port
   c = &c[i+1];
   for(i=0;c[i]!=',' && c[i]!='\n' && c[i]!=' ' && c[i]!='\r';i++) gServer[s].port[i]=c[i];
   gServer[s].port[i]='\0';
  }
  if (c[i]==',') //there is a label
  {
   //get label
   c = &c[i+2];
   for(i=0;c[i]!='\n' && c[i]!='\r';i++) gServer[s].label[i]=c[i];
   gServer[s].label[i]='\0';
  } else gServer[s].label[0] = '\0';
  //is this the default server?
  if (strcmp(gServer[s].name,gTempServer.name)==0 && strcmp(gServer[s].port,gTempServer.port)==0) gDefaultServer=s;
 }
 return 0;
}

inline int ini_write()
{
 int i;

 clearerr(fp);

 /* TODO */
 /* write comments here */

 fprintf(fp,"[global]\n");

 fprintf(fp,"defaultserver= %s",gServer[gDefaultServer].name);
 if (gServer[gDefaultServer].port[0] != '\0') fprintf(fp,":%s",gServer[gDefaultServer].port); fprintf(fp,"\n");
 fprintf(fp,"autoconnect= %1d\n",gDefaultServerAutoconnect);

 fprintf(fp,"\n");

 fprintf(fp,"[serverlist]\n");
 for (i=0;i<MAX_SERVERS;i++)
 {
  if (gServer[i].name[0] == '\0') continue;
  fprintf(fp,"server%1d= %s",i,gServer[i].name);
  if (gServer[i].port[0] != '\0') fprintf(fp,":%s",gServer[i].port);
  if (gServer[i].label[0] != '\0') fprintf(fp,", %s",gServer[i].label);
  fprintf(fp,"\n");
 }

 return ferror(fp)?-1:0;
}

int iniOpen(char *filename)
{
 int ret;
 fp = fopen(filename,"rt");
 if (fp==NULL)
 {
  perror("ini_open()");
  return -1; 
 }
 ret = ini_read();
 fclose(fp);
 gModified = false;

 return ret;
}

int iniOpen()
{
 int ret;
 fp = fopen(gDefaultFilename,"rt");
 if (fp==NULL)
 {
  perror("ini_open()");
  return -1;
 }
 ret = ini_read();
 fclose(fp);

 gModified = false;

 return ret;
}

int iniClose()
{
 int ret;
 if (!gModified) return 0;
 fp = fopen(gDefaultFilename,"wt");
 if (fp==NULL)
 {
  perror("iniClose()");
  return -1;
 }
 ret = ini_write();
 fclose(fp);
 return ret; 
}

int iniAddServer(const ServerType server)
{
 //Add a server in the first empty slot
 int i;
 for(i=0;i<MAX_SERVERS;i++) if (gServer[i].name[0]=='\0')
 {
  strcpy(gServer[i].name,server.name);
  strcpy(gServer[i].port,server.port);
  strcpy(gServer[i].label,server.label);
  gServer[i].autoconnect = server.autoconnect;
  gModified = true;
  return 0;
 }
 return -1;
}

int iniRemoveServer(int n)
{
 int i;
 if (n >= MAX_SERVERS) return -1;
 for(i=n;i<MAX_SERVERS-1;i++) gServer[i] = gServer[i+1];
 gServer[MAX_SERVERS-1].name[0]='\0';
 gServer[MAX_SERVERS-1].port[0]='\0';
 gServer[MAX_SERVERS-1].label[0]='\0';
 gServer[MAX_SERVERS-1].autoconnect = false;
 if (n==gDefaultServer)
 {//Select the first server and set autoconnect to false
  gDefaultServer = 0;
  gDefaultServerAutoconnect = false; 
 }
 gModified = true;
 return 0;
}

int iniSetDefaultServer(int n, bool autoconnect)
{
 if (n>=MAX_SERVERS) return -1;
 gDefaultServer = n;
 gDefaultServerAutoconnect = autoconnect;
 gModified = true;
 return 0;
}
